/* billard3d.c
**
**    drawing all with OpenGL
**    Copyright (C) 2001  Florian Berger
**    Email: harpin_floh@yahoo.de, florian.berger@jk.uni-linz.ac.at
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License Version 2 as
**    published by the Free Software Foundation;
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <GL/glut.h>
//#include <sys/timeb.h>  /* my_time */
#include <sys/time.h>   // us time measure
#include "billard.h"
#include "ball.h"
#include "table.h"
#include "queue.h"
#include "png_loader.h"
#include "aiplayer.h"
#include "options.h"

#ifdef XMESA
#include <GL/xmesa.h>
#endif


#define LIT 1
#define TEXTURED 2
#define REFLECT 3
#define ANIMATE 10
#define POINT_FILTER 20
#define LINEAR_FILTER 21
#define QUIT 100

//#define WIDTH   1280
//#define HEIGHT  1024
//#define WIDTH   1024
//#define HEIGHT   768
int win_width = 800;
int win_height = 600;

#define WIDTH    win_width
#define HEIGHT   win_height

BallsType balls;
BordersType walls;


static int frametime_ms = 20;


static int fullscreen=1;

static GLuint table_obj = 0;
static GLboolean Animate = GL_TRUE;

static GLfloat Xrot = 0.0, Yrot = 0.0, Zrot = 0.0;
static GLfloat Xque = 0.0, Yque = 0.0, Zque = 0.0;
static GLfloat Xrot_offs=0.0, Yrot_offs=0.0, Zrot_offs=0.0;
static GLfloat scale = 1.0;



int moving, start_x, start_y;
int scaling, scaling_start, scaling_start2;
GLfloat cam_dist_aim = 0.8;
GLfloat cam_dist;
GLfloat cam_FOV=40.0;

static int  vline_on=1;
static int  key_modifiers;
static int  queue_view=1;
static int  queue_anim=0;
static GLfloat queue_offs=0.06;
#define queue_point_x  player[act_player].cue_x
#define queue_point_y  player[act_player].cue_y
#define queue_strength player[act_player].strength

static int  balls_moving=0;

/* reflection map */
static int    spheretexw,spheretexh;
static char * spheretexdata;
static GLuint spheretexbind;
static char * lightspheretexdata;
static GLuint lightspheretexbind;


static int  show_fps=0;

static int  stereo_rg_on=0;

VMvect comp_dir;

static int  human_human_mode=0;
static int  act_player=0;   /* 0 or 1 */
static char * player_names[]={"Human Player","AI Player","Human Player 2","AI Player 2"};
static char * half_full_names[]={"any","full","half"};

static struct Player{
    int is_AI;
    int half_full;
    int queue_view;
    double Zque;
    double Xque;
    double cue_x;
    double cue_y;
    double strength;
    char *name;
} player[2];

//#define Xque (player[act_player].X_que)
//#define Zque (player[act_player].Z_que)
//#define queue_view (player[act_player].queueview)


/*
int my_time(void)
{struct timeb ts;
      ftime(&ts);
      return(ts.time*1000+ts.millitm);
}*/

double angle_pm180(double ang)
{
    while ( ang >  180.0 ) ang-=360.0;
    while ( ang < -180.0 ) ang+=360.0;
    return ang;
}


void toggle_queue_view()
{
    queue_view = (queue_view==0)?1:0;
    if( queue_view ){
        Xrot_offs=angle_pm180(Xrot-Xque);  Xrot=Xque;
        Zrot_offs=angle_pm180(Zrot-Zque);  Zrot=Zque;
    }
}

void init_players()
{
    player[0].is_AI=0;
    player[1].is_AI=human_human_mode?0:1;
    player[0].half_full=BALL_ANY;
    player[1].half_full=BALL_ANY;
    player[0].name=player_names[0];
    player[1].name=player_names[human_human_mode?2:1];
    player[0].Xque=-60.0;
    player[1].Xque=-83.0;
    player[0].Zque=0.0;
    player[1].Zque=0.0;
    player[0].cue_x=0.0;
    player[1].cue_x=0.0;
    player[0].cue_y=0.0;
    player[1].cue_y=0.0;
    player[0].strength=1.0;
    player[1].strength=1.0;
    player[0].queue_view=1;
    player[1].queue_view=human_human_mode?1:0;
}

int time_us()
// gives back time in microseconds
{
    struct timeval tv;
    struct timezone tz;
    tz.tz_minuteswest = 0;
    tz.tz_dsttime     = 0;
    gettimeofday(&tv,&tz);
    return ( tv.tv_sec*1000000+tv.tv_usec );
}


void queue_shot()
{
    VMvect dir, nx, ny, hitpoint;

    dir = vec_xyz(sin(Zque*M_PI/180.0)*sin(Xque*M_PI/180.0),
                  cos(Zque*M_PI/180.0)*sin(Xque*M_PI/180.0),
                  cos(Xque*M_PI/180.0));
    nx = vec_unit(vec_cross(vec_ez(),dir));  /* parallel to table */
    ny = vec_unit(vec_cross(nx,dir));        /* orthogonal to dir and nx */
    hitpoint = vec_add(vec_scale(nx,queue_point_x),vec_scale(ny,queue_point_y));
//    fprintf(stderr,"queue_shot: Zque=%f\n",Zque);
//    Zque=-137.020020;
    balls.ball[0].v.x =  -2.0*queue_strength*sin(Xque*M_PI/180.0)*sin(Zque*M_PI/180.0);
    balls.ball[0].v.y =  -2.0*queue_strength*sin(Xque*M_PI/180.0)*cos(Zque*M_PI/180.0);
//    balls.ball[0].w.x = -2.0/balls.ball[0].d/2.0*balls.ball[0].v.y;
//    balls.ball[0].w.y = +2.0/balls.ball[0].d/2.0*balls.ball[0].v.x;
//    balls.ball[0].w.z = -2.0/balls.ball[0].d/2.0;
    balls.ball[0].w = vec_scale(vec_cross(dir,hitpoint),3000.0);

    /* reset offset parameters */
    queue_point_x=0.0;
    queue_point_y=0.0;
}


void do_computer_move( int doit )
{
    VMvect dir;

    fprintf(stderr,"do_computermove: begin ai_get_strike_dir\n");

    dir = ai_get_stroke_dir(&balls,&walls,player[act_player].half_full);

    fprintf(stderr,"do_computermove: end ai_get_strike_dir\n");

    Zque = atan2(dir.x,dir.y)/M_PI*180.0;
    if(doit){
        if(!queue_view){
            queue_anim=30;
        } else {
            queue_shot();
        }
    }
    comp_dir=dir;
}


void evaluate_last_move()
{
    int out_half=0;
    int out_full=0;
    int nextplayer=1;
    /* if balls_out_half!=balls_out_full && human_full_half==BALL_ANY then  */

    out_half = BM_get_balls_out_half();
    out_full = BM_get_balls_out_full();

    nextplayer=1;

    if( player[act_player].half_full == BALL_ANY ){
        if(out_half>out_full){
            player[act_player].half_full=BALL_HALF;
            player[!act_player].half_full=BALL_FULL;
            nextplayer=0;
        }
        if(out_half<out_full){
            player[act_player].half_full=BALL_FULL;
            player[!act_player].half_full=BALL_HALF;
            nextplayer=0;
        }
    }

    /* wenn eigene rein naechster */
    if( player[act_player].half_full==BALL_HALF && out_half!=0 ){
        nextplayer=0;
    }
    if( player[act_player].half_full==BALL_FULL && out_full!=0 ){
        nextplayer=0;
    }

    /* wenn weisse rein */
    if( BM_get_white_out() ){
        nextplayer=1;
        balls.ball[0].in_game=1;
        balls.ball[0].v=vec_xyz(0.0,0.0,0.0);
        balls.ball[0].w=vec_xyz(0.0,0.0,0.0);
        balls.ball[0].r=vec_xyz(0.0,-TABLE_L/4.0,0.0);
    }

    BM_reset_move_info();

    if(nextplayer){
//        Xque=player[act_player].Xque;
//        player[act_player].Zque=Zque;
        player[act_player].queue_view=queue_view;
        act_player = (act_player==1) ? 0 : 1 ;
//        Xque=player[act_player].Xque;
        Xque=player[act_player].Xque;
        queue_view=player[act_player].queue_view;
    }
}


static void Idle( void ) /*FOLD00*/
{
    int i;
//    static int balls_moving=0;
    static int balls_were_moving=0;
    Xrot_offs *= 0.85;
    Zrot_offs *= 0.85;

   if (Animate) {
//      Xrot += DXrot;
//      Yrot += DYrot;
       balls_were_moving = balls_moving;
       for(i=0;i<2;i++){
           balls_moving = proceed_dt( &balls, &walls, 0.01 );
       }
       if( (!balls_moving) && balls_were_moving ){
           evaluate_last_move();
           if( player[act_player].is_AI ){
               do_computer_move(1);
           }
       }
//       balls_moving = 1;
       if ( queue_anim > 0 ){
           queue_anim--;
           queue_offs=0.16+( 0.16*(double)(30-queue_anim)*(double)(30-queue_anim)/30.0/30.0*sin((double)(30-queue_anim)/3.0) );
           if( queue_anim==0 ){
               queue_shot();
               queue_offs=0.06;
           }
       }
      glutPostRedisplay();
   } else {
       glutPostRedisplay();
   }
}




static void Idle_timer( int value )
{
    int i;
//    static int balls_moving=0;
    static int balls_were_moving=0;
    Xrot_offs *= 0.85;
    Zrot_offs *= 0.85;


    glutTimerFunc( frametime_ms, Idle_timer, value );  /* assure a framerate of max 50 fps */

   if (Animate) {
//      Xrot += DXrot;
//      Yrot += DYrot;
       balls_were_moving = balls_moving;
//       for(i=0;i<(frametime_ms+5)/10;i++){   /* assure constant time flow */
       for(i=0;i<(frametime_ms+5)/7;i++){   /* assure constant time flow */
           balls_moving = proceed_dt( &balls, &walls, 0.01 );
       }
       if( (!balls_moving) && balls_were_moving ){
           evaluate_last_move();
           if( player[act_player].is_AI ){
               do_computer_move(1);
           }
       }
//       balls_moving = 1;
       if ( queue_anim > 0 ){
           queue_anim--;
           queue_offs=0.16+( 0.16*(double)(30-queue_anim)*(double)(30-queue_anim)/30.0/30.0*sin((double)(30-queue_anim)/3.0) );
           if( queue_anim==0 ){
               queue_shot();
               queue_offs=0.06;
           }
       }
      glutPostRedisplay();
   } else {
       glutPostRedisplay();
   }
}




static void
MouseEvent(int button, int state, int x, int y)
{
    key_modifiers=glutGetModifiers();
    if (button == GLUT_LEFT_BUTTON) {
        if (state == GLUT_DOWN) {
            moving = 1;
            start_x = x;
            start_y = y;
        }
        if (state == GLUT_UP) {
            moving = 0;
        }
    }
    if (button == GLUT_RIGHT_BUTTON){
        if (state == GLUT_DOWN) {
            scaling = 1;
            scaling_start = y;
            scaling_start2 = x;
        }
        if (state == GLUT_UP) {
            scaling = 0;
        }
    }
    if (button == GLUT_MIDDLE_BUTTON) {
        if(!queue_view){
            queue_anim=30;
        } else {
            queue_shot();
        }
    }
    glutPostRedisplay();
}


static void
MouseMotion(int x, int y)
{
    static double acc;

    if(moving && scaling){
//        cam_look_offs = vec_add(cam_look_offs, )
    } else if(moving){
        if( key_modifiers & GLUT_ACTIVE_CTRL ) acc=0.02; else acc=1.0;
        Xrot +=  (double)(y-start_y)*acc;
        Zrot +=  (double)(x-start_x)*acc;
        start_x = x;
        start_y = y;
        if( queue_view ){
            Xque=Xrot;
            Zque=Zrot;
        }
    } else if(scaling){
        if( key_modifiers & GLUT_ACTIVE_CTRL ){
            double old_FOV;
            old_FOV=cam_FOV;
            cam_FOV  +=  (y-scaling_start)*0.05;
            if( cam_FOV<10.0 ) cam_FOV=10.0;
            if( cam_FOV>110.0 ) cam_FOV=110.0;
            cam_dist_aim = cam_dist_aim*tan(old_FOV*M_PI/360.0)/tan(cam_FOV*M_PI/360.0);
            cam_dist = cam_dist_aim;
            scaling_start = y;
        } else if( key_modifiers & GLUT_ACTIVE_SHIFT ){
            queue_point_y +=  (y-scaling_start)*0.0005;
            queue_point_x +=  (x-scaling_start2)*0.0005;
//            fprintf(stderr,"queue_point_x: %f\n",queue_point_x);
//            fprintf(stderr,"queue_point_y: %f\n",queue_point_y);
//            if( cam_dist_aim<0.0 ) cam_dist_aim=0.0;
            scaling_start = y;
            scaling_start2 = x;
        } else {
            cam_dist_aim +=  (y-scaling_start)*0.005;
            if( cam_dist_aim<0.0 ) cam_dist_aim=0.0;
            scaling_start = y;
        }
    }
    glutPostRedisplay();
}


void myRect2D( double x1, double y1, double x2, double y2 )
{
    glBegin(GL_QUADS);
    glVertex3f(x1,y1,-0.5);
    glVertex3f(x1,y2,-0.5);
    glVertex3f(x2,y2,-0.5);
    glVertex3f(x2,y1,-0.5);
    glEnd();
}


static void DisplayFunc( void )
{
//   int i;
   double th,ph;
   GLfloat light_position[] = { 0.0, 0.0, 180.0, 0.0 };
   GLfloat light_diff[]     = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat light_amb[]      = { 0.7, 0.7, 0.7, 1.0 };
   GLfloat light_diff_r[]   = { 1.0, 0.0, 0.0, 1.0 };
   GLfloat light_amb_r[]    = { 0.5, 0.0, 0.0, 1.0 };
   GLfloat light_diff_g[]   = { 0.0, 0.7, 0.0, 1.0 };
   GLfloat light_amb_g[]    = { 0.0, 0.5*0.7, 0.0, 1.0 };
   myvec cam_pos;
   static GLfloat real_dist=0.0;
   int t_act;
   static int dt;
   static int t_prev;
   static int count=0;
   static double fps;
   int i;
   int act_buffer;

   static GLfloat rg_eye_dist=0.06;
   static GLfloat rg_scrw=0.225;
   static GLfloat rg_scrh=0.165;

   if(show_fps){
       count++;
       t_act  = time_us();
       dt     += t_act-t_prev;
       t_prev = t_act;
       if(count==5){
           count=0;
           fps=5.0/(double)dt*1000000.0;
//                      fprintf(stderr,"fps=%f\n",fps);
           dt=0;
           frametime_ms=(int)(dt/1000.0);
           if( frametime_ms<20 ) frametime_ms=20;
//           if( frametime_ms>60 ) frametime_ms=60;
       }
   }

   cam_dist = (cam_dist*0.94)+(cam_dist_aim+vec_abs(balls.ball[0].v)*0.4)*0.06;

   glFogf (GL_FOG_START, (cam_dist/2.0>cam_dist-1.0) ? cam_dist/2.0 : cam_dist-1.0 );
   glFogf (GL_FOG_END, cam_dist+6.0);

   real_dist = cam_dist;

   glGetIntegerv(GL_DRAW_BUFFER, &act_buffer);
   for(i=0;i<=stereo_rg_on;i++){
   glMatrixMode( GL_PROJECTION );
   if       (i==0 && stereo_rg_on){
       glDrawBuffer(GL_AUX0);
       glReadBuffer(GL_AUX0);
       glLoadIdentity();
       glFrustum( (-rg_scrw-rg_eye_dist)/2.0, (rg_scrw-rg_eye_dist)/2.0,
                  -rg_scrh/2.0, rg_scrh/2.0, 0.30, 5.0);
   }else if (i==1 && stereo_rg_on){
       glDrawBuffer(GL_AUX1);
       glReadBuffer(GL_AUX1);
       glLoadIdentity();
       glFrustum( (-rg_scrw+rg_eye_dist)/2.0, (rg_scrw+rg_eye_dist)/2.0,
                  -rg_scrh/2.0, rg_scrh/2.0, 0.30, 5.0);
   }else{
//       glFrustum( left, right, bottom, top, zNear, zFar );
       glLoadIdentity();
       glFrustum( -0.1*tan(cam_FOV*M_PI/360.0), 0.1*tan(cam_FOV*M_PI/360.0),
                  -0.1*tan(cam_FOV*M_PI/360.0)*(double)win_height/(double)win_width,
                  +0.1*tan(cam_FOV*M_PI/360.0)*(double)win_height/(double)win_width, 0.1, 10.0);
   }
   glMatrixMode( GL_MODELVIEW );
   glLoadIdentity();

   if       (i==0 && stereo_rg_on){
       glTranslatef( -rg_eye_dist/2.0, 0.0, 0.0 );
   }else if (i==1 && stereo_rg_on){
       glTranslatef( rg_eye_dist/2.0, 0.0, 0.0 );
   }else{
   }
       glTranslatef( 0.0, 0.0, -real_dist );

   if      (i==0 && stereo_rg_on){
       glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diff_g);
       glLightfv(GL_LIGHT0, GL_AMBIENT,  light_amb_g);
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
   }else if(i==1 && stereo_rg_on){
       glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diff_r);
       glLightfv(GL_LIGHT0, GL_AMBIENT,  light_amb_r);
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
//       glEnable(GL_BLEND);
//       glBlendFunc (GL_ONE, GL_ONE);
   } else {
       glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diff);
       glLightfv(GL_LIGHT0, GL_AMBIENT,  light_amb);
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
//       glMaterialfv(GL_FRONT, GL_DIFFUSE,  col_black);
//       glRectf( -1.0,-1.0,1.0,1.0 );
//       glClear( GL_DEPTH_BUFFER_BIT );
   }

   glPushMatrix();

   glRotatef(Xrot+Xrot_offs, 1.0, 0.0, 0.0);
   glRotatef(Yrot+Yrot_offs, 0.0, 1.0, 0.0);
   glRotatef(Zrot+Zrot_offs, 0.0, 0.0, 1.0);
   glTranslatef( -balls.ball[0].r.x/**100.0*/,
                 -balls.ball[0].r.y/**100.0*/,
                 -balls.ball[0].r.z/**100.0*/ );

   glLightfv(GL_LIGHT0, GL_POSITION, light_position);

   /* draw table */
   glCallList(table_obj);

//   glScalef(100.0,100.0,100.0);

   th=Xrot/180.0*M_PI;
   ph=Zrot/180.0*M_PI;
   cam_pos=vec_scale(vec_xyz(sin(th)*sin(ph),sin(th)*cos(ph),cos(th)),
                     real_dist);
   cam_pos=vec_add(cam_pos,balls.ball[0].r);
//   my_glBox(cam_pos.x+0.01, cam_pos.y+0.01, cam_pos.z+0.01,
//            cam_pos.x-0.01, cam_pos.y-0.01, cam_pos.z-0.01);


   /* draw balls with reflections and shadows */
   if(options_ball_reflections_blended){
       draw_balls(balls,cam_pos,cam_FOV,win_width,spheretexbind);
   } else {
       draw_balls(balls,cam_pos,cam_FOV,win_width,lightspheretexbind);
   }


   if( !queue_view && !balls_moving ){  /* draw queue */
/*       double dx,dy,dz,th,ph;
       ph=Zque/180.0*M_PI;
       th=Xque/180.0*M_PI;
       dy=+1.0*sin(th)*cos(ph);
       dx=+1.0*sin(th)*sin(ph);
       dz=+1.0*cos(th);
       glBegin( GL_LINES );
       glVertex3f(balls.ball[0].r.x,balls.ball[0].r.y,balls.ball[0].r.z);
       glVertex3f(balls.ball[0].r.x+dx,balls.ball[0].r.y+dy,balls.ball[0].r.z+dz);
       glEnd();*/
/*       glBegin( GL_LINES );
       glVertex3f(balls.ball[0].r.x,balls.ball[0].r.y,balls.ball[0].r.z);
       glVertex3f(balls.ball[0].r.x+comp_dir.x,balls.ball[0].r.y+comp_dir.y,balls.ball[0].r.z+comp_dir.z);
       glEnd();*/
       draw_queue( balls.ball[0].r, Xque, Zque, queue_offs,
                   queue_point_x, queue_point_y,
                   spheretexbind );
   }

   if( vline_on && queue_view && !balls_moving ){
       glBegin( GL_LINES );
       glVertex3f(balls.ball[0].r.x,balls.ball[0].r.y,balls.ball[0].r.z);
       glVertex3f(balls.ball[0].r.x,balls.ball[0].r.y,balls.ball[0].r.z+1.0);
       glEnd();
   }

   if(1){   /* player text */
       char str[200];
       sprintf(str,"%s - %s",player[act_player].name,half_full_names[player[act_player].half_full]);
       glMatrixMode( GL_PROJECTION );
       glPushMatrix();
       glLoadIdentity();

       glMatrixMode( GL_MODELVIEW );
       glPushMatrix();
       glLoadIdentity();
       glColor3f(0.5,1.0,0.7);
       glDisable(GL_LIGHTING);
       glDisable(GL_TEXTURE_2D);
       glDisable(GL_DEPTH_TEST);
//       glRasterPos3f(-0.35,-0.27,-1.2);
//       glRasterPos3f(-0.9,-0.9*(double)win_height/(double)win_width,-0.5);
       glRasterPos3f(-0.9,-0.9,-0.5);

       for(i=0;str[i]!=0;i++){
            glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,str[i]);
//            glutBitmapCharacter(GLUT_BITMAP_8_BY_13,str[i]);
       }

       if(show_fps){
           sprintf(str,"fps=%f",fps);
           glRasterPos3f(0.5,-0.9,-0.5);
           for(i=0;str[i]!=0;i++){
               glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,str[i]);
           }
       }

       /* strength bar */
       glColor3f(0.5,1.0,0.7);
       myRect2D( -0.5, -0.800,  0.5,                -0.770 );
       glColor3f(0.8,0.0,0.0);
       myRect2D( -0.5, -0.795, -0.5+queue_strength, -0.775 );

       glEnable(GL_DEPTH_TEST);
       glEnable(GL_LIGHTING);
       glEnable(GL_TEXTURE_2D);
       glPopMatrix();

       glMatrixMode( GL_PROJECTION );
       glPopMatrix();
       glMatrixMode( GL_MODELVIEW );
   }

   glPopMatrix();

   } // for RG-frames

/*   if(stereo_rg_on) {
       glDrawBuffer(act_buffer);
       glReadBuffer(GL_AUX1);
       glCopyPixels(0,0,WIDTH,HEIGHT,GL_COLOR);
   }*/

   glutSwapBuffers();
}


static void Reshape( int width, int height ) /*FOLD00*/
{
/*   if (width > WIDTH)
       width = WIDTH;
   if (height > HEIGHT)
       height = HEIGHT;*/

    win_width=width;
    win_height=height;
   glViewport( 0, 0, width, height );
   glMatrixMode( GL_PROJECTION );
   glLoadIdentity();
//   glFrustum( -1.0, 1.0, -1.0, 1.0, 4.0, 1000.0 );
   gluPerspective(cam_FOV, /* field of view in degree */
     (GLfloat) width/(GLfloat) height, /* aspect ratio */
     0.1, /* Z near */
     10.0); /* Z far */
//   gluLookAt(0.0, 1.0, 380.0,  /* eye is at (0,8,60) */
//     0.0, 0.0, 0.0,      /* center is at (0,8,0) */
//     0.0, 1.0, 0.0);      /* up is in postivie Y direction */
}


static void SetMode(GLuint m) /*FOLD00*/
{
   /* disable everything */
   glDisable(GL_LIGHTING);
   glDisable(GL_TEXTURE_2D);
   glDisable(GL_TEXTURE_GEN_S);
   glDisable(GL_TEXTURE_GEN_T);

   /* enable what's needed */
   if (m==LIT) {
      glEnable(GL_LIGHTING);
   }
   else if (m==TEXTURED) {
      glEnable(GL_LIGHTING);
      glEnable(GL_TEXTURE_2D);
   }
   else if (m==REFLECT) {
      glEnable(GL_LIGHTING);
      glEnable(GL_TEXTURE_2D);
      glEnable(GL_TEXTURE_GEN_S);
      glEnable(GL_TEXTURE_GEN_T);
   }
}


static void ModeMenu(int entry) /*FOLD00*/
{
   if (entry==ANIMATE) {
      Animate = !Animate;
   }
   else if (entry==POINT_FILTER) {
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   }
   else if (entry==LINEAR_FILTER) {
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
   }
   else if (entry==QUIT) {
      exit(0);
   }
   else {
      SetMode(entry);
   }
   glutPostRedisplay();
}


static void Key( unsigned char key, int x, int y ) /*FOLD00*/
{
   switch (key) {
      case 27:
         exit(0);
         break;
      case ' ':
      case 13:
          if(!queue_view){
              queue_anim=30;
          } else {
              queue_shot();
          }
         break;
      case '0':
          do_computer_move(0);
         break;
      case 'a':
          player[act_player].is_AI = !player[act_player].is_AI;
         break;
      case 'n':
          player[0].half_full=BALL_ANY;
          player[1].half_full=BALL_ANY;
          create_8ball_scene( &balls, &walls );
         break;
      case 'b':
          show_fps = (show_fps==0)?1:0;
         break;
      case 'v':
          vline_on = (vline_on==0)?1:0;
         break;
      case 'c':
          toggle_queue_view();
         break;
      case 's':
          stereo_rg_on = (stereo_rg_on==0)?1:0;
         break;
#ifdef XMESA
       case 'f':
           fullscreen=(!fullscreen);
           XMesaSetFXmode(fullscreen ? XMESA_FX_FULLSCREEN : XMESA_FX_WINDOW);
           break;
#endif
      case 'r':
          options_ball_reflections_blended=!options_ball_reflections_blended;
         break;
   }
   glutPostRedisplay();
}


double strength01( double value )
{
    if( value > 1.0 ) value=1.0;
    if( value < 0.0 ) value=0.0;
    return value;
}


static void SpecialKey( int key, int x, int y ) /*FOLD00*/
{
   float step = 3.0;

   switch (key) {
      case GLUT_KEY_PAGE_UP:
      case GLUT_KEY_UP:
          queue_strength = strength01( queue_strength+0.01 );
//         Xrot += step;
         break;
      case GLUT_KEY_PAGE_DOWN:
      case GLUT_KEY_DOWN:
          queue_strength = strength01( queue_strength-0.01 );
//         Xrot -= step;
         break;
      case GLUT_KEY_LEFT:
         Zrot += step;
         break;
      case GLUT_KEY_RIGHT:
         Zrot -= step;
         break;
      case 'x':
         scale *= 1.1;
         break;
      case 'y':
         scale /= 1.1;
         break;
   }
   glutPostRedisplay();
}


static void Init( void ) /*FOLD00*/
{
//    int i;
    GLfloat fogColor[4] = {0.0, 0.0, 0.0, 1.0};
    int    depth;

    init_players();

    if ( options_dither )
        glEnable(GL_DITHER);
    else
        glDisable(GL_DITHER);

    glActiveTextureARB(GL_TEXTURE1_ARB);
    glEnable(GL_TEXTURE_2D);
    glActiveTextureARB(GL_TEXTURE0_ARB);
    glEnable(GL_TEXTURE_2D);


    glGenTextures(1,&spheretexbind);
    load_png("sphere_map_128x128.png",&spheretexw,&spheretexh,&depth,&spheretexdata);
    glBindTexture(GL_TEXTURE_2D,spheretexbind);
    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, spheretexw, spheretexh, GL_RGB,
                      GL_UNSIGNED_BYTE, spheretexdata);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, options_tex_min_filter);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, options_tex_mag_filter);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    free(spheretexdata);
    fprintf(stderr,"spheretexbind=%d\n",spheretexbind);

    lightspheretexdata=0;
    glGenTextures(1,&lightspheretexbind);
    load_png("sphere_map_128x128_light.png",&spheretexw,&spheretexh,&depth,&lightspheretexdata);
    glBindTexture(GL_TEXTURE_2D,lightspheretexbind);
    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, spheretexw, spheretexh, GL_RGB,
                      GL_UNSIGNED_BYTE, lightspheretexdata);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, options_tex_min_filter);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, options_tex_mag_filter);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    fprintf(stderr,"lightspheretexbind=%d\n",lightspheretexbind);
    free(lightspheretexdata);


    glEnable(GL_FOG);
    glFogi (GL_FOG_MODE, GL_LINEAR);
    glHint (GL_FOG_HINT, GL_FASTEST);
    glFogf (GL_FOG_START, 0.0);
    glFogf (GL_FOG_END, 9.0);
    glFogfv (GL_FOG_COLOR, fogColor);

    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
//    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);


/*    glEnable (GL_LINE_SMOOTH);
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
    glLineWidth (1.5);
    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE); */
//    glShadeModel (GL_FLAT);

    create_8ball_scene( &balls, &walls );


    table_obj = create_table(spheretexbind, &walls);

    /* lighting */
//    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
//    glLightfv(GL_LIGHT0, GL_AMBIENT, amb);
//    glLightfv(GL_LIGHT0, GL_DIFFUSE, white);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);


   fprintf(stderr,"enabling depth test\n");
   glEnable(GL_DEPTH_TEST);
   glFrontFace(GL_CW);
//   glCullFace(GL_FRONT);
   glEnable(GL_CULL_FACE);
   glDepthMask(GL_TRUE);
//   glDepthFunc( GL_GREATER );
//   glDepthFunc( GL_LESS );
   glDepthFunc( GL_LEQUAL );
//   glDepthFunc( GL_ALWAYS );
//   glDepthFunc( GL_EQUAL );

   SetMode(REFLECT);
}


int main( int argc, char *argv[] ) /*FOLD00*/
{

    int auxnr;

   /* Tell Mesa GLX to use 3Dfx driver in fullscreen mode. */
//   putenv("MESA_GLX_FX=fullscreen");

   /* Disable 3Dfx Glide splash screen */
//   putenv("FX_GLIDE_NO_SPLASH=");


   glutInitWindowPosition(0, 0);
   //   glutInitWindowSize( 800, 800 );
   glutInitWindowSize(WIDTH, HEIGHT);

   glutInit( &argc, argv );

   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   //   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE );

   glutCreateWindow( argv[0] );

   human_human_mode=0;
   if( argc>1 && argv[1][0]=='2' ) human_human_mode=1;

//   glutFullScreen();

   Init();

   glGetIntegerv(GL_AUX_BUFFERS, &auxnr);
   fprintf(stderr,"# of AUX-buffers:%d\n",auxnr);

   glEnable(GL_LIGHTING);
   glutReshapeFunc( Reshape );
   glutKeyboardFunc( Key );
   glutSpecialFunc( SpecialKey );
   glutDisplayFunc( DisplayFunc );
//   glutIdleFunc( Idle );
   glutTimerFunc( frametime_ms, Idle_timer, 10 );  /* assure a framerate of max 50 fps (1frame/20ms) */
   glutMouseFunc(MouseEvent);
//   glutPassiveMotionFunc( MouseMotion );
   glutMotionFunc(MouseMotion);

/* glutCreateMenu(ModeMenu);
   glutAddMenuEntry("Lit", LIT);
   glutAddMenuEntry("Textured", TEXTURED);
   glutAddMenuEntry("Reflect", REFLECT);
   glutAddMenuEntry("Point Filtered", POINT_FILTER);
   glutAddMenuEntry("Linear Filtered", LINEAR_FILTER);
   glutAddMenuEntry("Toggle Animation", ANIMATE);
   glutAddMenuEntry("Quit", QUIT);
   glutAttachMenu(GLUT_RIGHT_BUTTON);*/

   glutMainLoop();
   return 0;
}
